require 'base64'

module SptyAPI
  class Authorization
    attr_accessor :access_token, :token_type, :expires_in, :scope,
                  :authorized, :errors

    ACCOUNT_API = 'https://accounts.spotify.com/api/'

    def initialize(type:)
      if type == 'client_credentials'
        self.class.via_client_credentials(self)
      end
    end

    def self.via_client_credentials(auth)
      auth.login
    end

    def login
      client_id  = SptyAPI.configuration.client_id
      secret_key = SptyAPI.configuration.secret_key
      auth_cred  = Base64.strict_encode64("#{client_id}:#{secret_key}")

      http = SptyAPI::HTTP::Client.new(
        http_method: 'post',
        token_type: 'Basic',
        access_token: auth_cred,
        endpoint: 'token'
      )

      http.api_overwrite          = ACCOUNT_API
      http.content_type_overwrite = 'application/x-www-form-urlencoded'
      http.body_parameter         = 'grant_type=client_credentials'

      res = JSON.parse(http.request.read_body)

      res['access_token'] ? authorize(res) : reject(res)
    end

    def client_is_authorized?
      authorized
    end

    def authorize(res)
      self.authorized   = true
      self.access_token = res['access_token']
      self.token_type   = res['token_type']
      self.expires_in   = Time.now + res['expires_in']
      self.scope        = res['scope']
    end

    def reject(res)
      self.authorized   = false
      self.errors       = res
    end

    private

    def setup_credentials

    end
  end
end
